@@ -54,13 +54,10 @@ static int iw_conn(struct nl80211_state *state,
 	argv++;
 	argc--;
 
-	ret = parse_keys(msg, argv, argc);
+	ret = parse_keys(msg, &argv, &argc);
 	if (ret)
 		return ret;
 
-	argc -= 4;
-	argv += 4;
-
 	if (!argc)
 		return 0;
 
@@ -157,7 +154,10 @@ static int iw_connect(struct nl80211_state *state,
 	 * Alas, the kernel doesn't do that (yet).
 	 */
 
-	__do_listen_events(state, ARRAY_SIZE(cmds), cmds, &printargs);
+	__do_listen_events(state,
+			   ARRAY_SIZE(cmds), cmds,
+			   ARRAY_SIZE(cmds), cmds,
+			   &printargs);
 	return 0;
 }
 TOPLEVEL(connect, "[-w] <SSID> [<freq in MHz>] [<bssid>] [key 0:abcde d:1:6162636465] [mfp:req/opt/no]",
@@ -228,7 +228,7 @@ static int iw_auth(struct nl80211_state *state,
 	argv++;
 	argc--;
 
-	return parse_keys(msg, argv, argc);
+	return parse_keys(msg, &argv, &argc);
  nla_put_failure:
 	return -ENOSPC;
 }
