@@ -122,17 +122,20 @@ static void parse_cqm_event(struct nlattr **attrs)
 
 	if (cqm[NL80211_ATTR_CQM_RSSI_THRESHOLD_EVENT]) {
 		enum nl80211_cqm_rssi_threshold_event rssi_event;
+		int32_t rssi_level = -1;
 		bool found_one = false;
 
 		rssi_event = nla_get_u32(cqm[NL80211_ATTR_CQM_RSSI_THRESHOLD_EVENT]);
+		if (cqm[NL80211_ATTR_CQM_RSSI_LEVEL])
+			rssi_level = nla_get_u32(cqm[NL80211_ATTR_CQM_RSSI_LEVEL]);
 
 		switch (rssi_event) {
 		case NL80211_CQM_RSSI_THRESHOLD_EVENT_HIGH:
-			printf("RSSI went above threshold\n");
+			printf("RSSI (%i dBm) went above threshold\n", rssi_level);
 			found_one = true;
 			break;
 		case NL80211_CQM_RSSI_THRESHOLD_EVENT_LOW:
-			printf("RSSI went below threshold\n");
+			printf("RSSI (%i dBm) went below threshold\n", rssi_level);
 			found_one = true;
 			break;
 		case NL80211_CQM_RSSI_BEACON_LOSS_EVENT:
@@ -293,6 +296,51 @@ static void parse_wowlan_wake_event(struct nlattr **attrs)
 		printf("\t* TCP connection ran out of tokens\n");
 }
 
+extern struct vendor_event *__start_vendor_event[];
+extern struct vendor_event *__stop_vendor_event;
+
+// Dummy to force the section to exist
+VENDOR_EVENT(0xffffffff, 0xffffffff, NULL);
+
+static void parse_vendor_event(struct nlattr **attrs, bool dump)
+{
+	__u32 vendor_id, subcmd;
+	unsigned int i;
+
+	if (!attrs[NL80211_ATTR_VENDOR_ID] ||
+	    !attrs[NL80211_ATTR_VENDOR_SUBCMD])
+		return;
+
+	vendor_id = nla_get_u32(attrs[NL80211_ATTR_VENDOR_ID]);
+	subcmd = nla_get_u32(attrs[NL80211_ATTR_VENDOR_SUBCMD]);
+
+	printf("vendor event %.6x:%d", vendor_id, subcmd);
+
+	for (i = 0; i < &__stop_vendor_event - __start_vendor_event; i++) {
+		struct vendor_event *ev = __start_vendor_event[i];
+
+		if (!ev)
+			continue;
+
+		if (ev->vendor_id != vendor_id)
+			continue;
+		if (ev->subcmd != subcmd)
+			continue;
+		if (!ev->callback)
+			continue;
+
+		ev->callback(vendor_id, subcmd, attrs[NL80211_ATTR_VENDOR_DATA]);
+		goto out;
+	}
+
+	if (dump && attrs[NL80211_ATTR_VENDOR_DATA])
+		iw_hexdump("vendor event",
+			   nla_data(attrs[NL80211_ATTR_VENDOR_DATA]),
+			   nla_len(attrs[NL80211_ATTR_VENDOR_DATA]));
+out:
+	printf("\n");
+}
+
 static void parse_nan_term(struct nlattr **attrs)
 {
 	struct nlattr *func[NL80211_NAN_FUNC_ATTR_MAX + 1];
@@ -650,6 +698,200 @@ static void parse_nan_match(struct nlattr **attrs)
 	printf("\n");
 }
 
+static void parse_new_peer_candidate(struct nlattr **attrs)
+{
+	char macbuf[ETH_ALEN * 3];
+	int32_t sig_dbm;
+
+	printf("new peer candidate");
+	if (attrs[NL80211_ATTR_MAC]) {
+		mac_addr_n2a(macbuf, nla_data(attrs[NL80211_ATTR_MAC]));
+		printf(" %s", macbuf);
+	}
+	if (attrs[NL80211_ATTR_RX_SIGNAL_DBM]) {
+		sig_dbm = nla_get_u32(attrs[NL80211_ATTR_RX_SIGNAL_DBM]);
+		printf(" %d dBm", sig_dbm);
+	}
+
+	printf("\n");
+}
+
+static void parse_recv_interface(struct nlattr **attrs, int command)
+{
+	switch (command) {
+	case NL80211_CMD_NEW_INTERFACE:
+		printf("new interface");
+		break;
+	case NL80211_CMD_DEL_INTERFACE:
+		printf("del interface");
+		break;
+	case NL80211_CMD_SET_INTERFACE:
+		printf("set interface");
+		break;
+	default:
+		printf("unknown interface command (%i) received\n", command);
+		return;
+	}
+
+	if (attrs[NL80211_ATTR_IFTYPE]) {
+		printf(" type ");
+		switch (nla_get_u32(attrs[NL80211_ATTR_IFTYPE])) {
+		case NL80211_IFTYPE_STATION:
+			printf("station");
+			break;
+		case NL80211_IFTYPE_AP:
+			printf("access point");
+			break;
+		case NL80211_IFTYPE_MESH_POINT:
+			printf("mesh point");
+			break;
+		case NL80211_IFTYPE_ADHOC:
+			printf("IBSS");
+			break;
+		case NL80211_IFTYPE_MONITOR:
+			printf("monitor");
+			break;
+		case NL80211_IFTYPE_AP_VLAN:
+			printf("AP-VLAN");
+			break;
+		case NL80211_IFTYPE_WDS:
+			printf("WDS");
+			break;
+		case NL80211_IFTYPE_P2P_CLIENT:
+			printf("P2P-client");
+			break;
+		case NL80211_IFTYPE_P2P_GO:
+			printf("P2P-GO");
+			break;
+		case NL80211_IFTYPE_P2P_DEVICE:
+			printf("P2P-Device");
+			break;
+		case NL80211_IFTYPE_OCB:
+			printf("OCB");
+			break;
+		case NL80211_IFTYPE_NAN:
+			printf("NAN");
+			break;
+		default:
+			printf("unknown (%d)",
+			       nla_get_u32(attrs[NL80211_ATTR_IFTYPE]));
+			break;
+		}
+	}
+
+	if (attrs[NL80211_ATTR_MESH_ID]) {
+		printf(" meshid ");
+		print_ssid_escaped(nla_len(attrs[NL80211_ATTR_MESH_ID]),
+				   nla_data(attrs[NL80211_ATTR_MESH_ID]));
+	}
+
+	if (attrs[NL80211_ATTR_4ADDR]) {
+		printf(" use 4addr %d", nla_get_u8(attrs[NL80211_ATTR_4ADDR]));
+	}
+
+	printf("\n");
+}
+
+static void parse_sta_opmode_changed(struct nlattr **attrs)
+{
+	char macbuf[ETH_ALEN*3];
+
+	printf("sta opmode changed");
+
+	if (attrs[NL80211_ATTR_MAC]) {
+		mac_addr_n2a(macbuf, nla_data(attrs[NL80211_ATTR_MAC]));
+		printf(" %s", macbuf);
+	}
+
+	if (attrs[NL80211_ATTR_SMPS_MODE])
+		printf(" smps mode %d", nla_get_u8(attrs[NL80211_ATTR_SMPS_MODE]));
+
+	if (attrs[NL80211_ATTR_CHANNEL_WIDTH])
+		printf(" chan width %d", nla_get_u8(attrs[NL80211_ATTR_CHANNEL_WIDTH]));
+
+	if (attrs[NL80211_ATTR_NSS])
+		printf(" nss %d", nla_get_u8(attrs[NL80211_ATTR_NSS]));
+
+	printf("\n");
+}
+
+static void parse_ch_switch_notify(struct nlattr **attrs, int command)
+{
+	switch (command) {
+	case NL80211_CMD_CH_SWITCH_STARTED_NOTIFY:
+		printf("channel switch started");
+		break;
+	case NL80211_CMD_CH_SWITCH_NOTIFY:
+		printf("channel switch");
+		break;
+	default:
+		printf("unknown channel switch command (%i) received\n", command);
+		return;
+	}
+
+	if (attrs[NL80211_ATTR_CH_SWITCH_COUNT])
+		printf(" (count=%d)", nla_get_u32(attrs[NL80211_ATTR_CH_SWITCH_COUNT]));
+
+	if (attrs[NL80211_ATTR_WIPHY_FREQ])
+		printf(" freq=%d", nla_get_u32(attrs[NL80211_ATTR_WIPHY_FREQ]));
+
+	if (attrs[NL80211_ATTR_CHANNEL_WIDTH]) {
+		printf(" width=");
+		switch(nla_get_u32(attrs[NL80211_ATTR_CHANNEL_WIDTH])) {
+		case NL80211_CHAN_WIDTH_20_NOHT:
+		case NL80211_CHAN_WIDTH_20:
+			printf("\"20 MHz\"");
+			break;
+		case NL80211_CHAN_WIDTH_40:
+			printf("\"40 MHz\"");
+			break;
+		case NL80211_CHAN_WIDTH_80:
+			printf("\"80 MHz\"");
+			break;
+		case NL80211_CHAN_WIDTH_80P80:
+			printf("\"80+80 MHz\"");
+			break;
+		case NL80211_CHAN_WIDTH_160:
+			printf("\"160 MHz\"");
+			break;
+		case NL80211_CHAN_WIDTH_5:
+			printf("\"5 MHz\"");
+			break;
+		case NL80211_CHAN_WIDTH_10:
+			printf("\"10 MHz\"");
+			break;
+		default:
+			printf("\"unknown\"");
+		}
+	}
+
+	if (attrs[NL80211_ATTR_WIPHY_CHANNEL_TYPE]) {
+		printf(" type=");
+		switch(nla_get_u32(attrs[NL80211_ATTR_WIPHY_CHANNEL_TYPE])) {
+		case NL80211_CHAN_NO_HT:
+			printf("\"No HT\"");
+			break;
+		case NL80211_CHAN_HT20:
+			printf("\"HT20\"");
+			break;
+		case NL80211_CHAN_HT40MINUS:
+			printf("\"HT40-\"");
+			break;
+		case NL80211_CHAN_HT40PLUS:
+			printf("\"HT40+\"");
+			break;
+		}
+	}
+
+	if (attrs[NL80211_ATTR_CENTER_FREQ1])
+		printf(" freq1=%d", nla_get_u32(attrs[NL80211_ATTR_CENTER_FREQ1]));
+
+	if (attrs[NL80211_ATTR_CENTER_FREQ2])
+		printf(" freq2=%d", nla_get_u32(attrs[NL80211_ATTR_CENTER_FREQ2]));
+
+	printf("\n");
+}
+
 static int print_event(struct nl_msg *msg, void *arg)
 {
 	struct genlmsghdr *gnlh = nlmsg_data(nlmsg_hdr(msg));
@@ -683,8 +925,11 @@ static int print_event(struct nl_msg *msg, void *arg)
 		  genlmsg_attrlen(gnlh, 0), NULL);
 
 	if (tb[NL80211_ATTR_IFINDEX] && tb[NL80211_ATTR_WIPHY]) {
-		if_indextoname(nla_get_u32(tb[NL80211_ATTR_IFINDEX]), ifname);
-		printf("%s (phy #%d): ", ifname, nla_get_u32(tb[NL80211_ATTR_WIPHY]));
+		/* if_indextoname may fails on delete interface/wiphy event */
+		if (if_indextoname(nla_get_u32(tb[NL80211_ATTR_IFINDEX]), ifname))
+			printf("%s (phy #%d): ", ifname, nla_get_u32(tb[NL80211_ATTR_WIPHY]));
+		else
+			printf("phy #%d: ", nla_get_u32(tb[NL80211_ATTR_WIPHY]));
 	} else if (tb[NL80211_ATTR_WDEV] && tb[NL80211_ATTR_WIPHY]) {
 		printf("wdev 0x%llx (phy #%d): ",
 			(unsigned long long)nla_get_u64(tb[NL80211_ATTR_WDEV]),
@@ -734,8 +979,12 @@ static int print_event(struct nl_msg *msg, void *arg)
 	case NL80211_CMD_SCHED_SCAN_RESULTS:
 		printf("got scheduled scan results\n");
 		break;
+	case NL80211_CMD_WIPHY_REG_CHANGE:
 	case NL80211_CMD_REG_CHANGE:
-		printf("regulatory domain change: ");
+		if (gnlh->cmd == NL80211_CMD_WIPHY_REG_CHANGE)
+			printf("regulatory domain change (phy): ");
+		else
+			printf("regulatory domain change: ");
 
 		reg_type = nla_get_u8(tb[NL80211_ATTR_REG_TYPE]);
 
@@ -906,6 +1155,11 @@ static int print_event(struct nl_msg *msg, void *arg)
 			nla_get_u32(tb[NL80211_ATTR_WIPHY_FREQ]),
 			(unsigned long long)nla_get_u64(tb[NL80211_ATTR_COOKIE]));
 		break;
+	case NL80211_CMD_FRAME_WAIT_CANCEL:
+		printf("frame wait cancel on freq %d (cookie %llx)\n",
+			nla_get_u32(tb[NL80211_ATTR_WIPHY_FREQ]),
+			(unsigned long long)nla_get_u64(tb[NL80211_ATTR_COOKIE]));
+		break;
 	case NL80211_CMD_NOTIFY_CQM:
 		parse_cqm_event(tb);
 		break;
@@ -934,13 +1188,7 @@ static int print_event(struct nl_msg *msg, void *arg)
 		       tb[NL80211_ATTR_ACK] ? "acked" : "no ack");
 		break;
 	case NL80211_CMD_VENDOR:
-		printf("vendor event %.6x:%d\n",
-			nla_get_u32(tb[NL80211_ATTR_VENDOR_ID]),
-			nla_get_u32(tb[NL80211_ATTR_VENDOR_SUBCMD]));
-		if (args->frame && tb[NL80211_ATTR_VENDOR_DATA])
-			iw_hexdump("vendor event",
-				   nla_data(tb[NL80211_ATTR_VENDOR_DATA]),
-				   nla_len(tb[NL80211_ATTR_VENDOR_DATA]));
+		parse_vendor_event(tb, args->frame);
 		break;
 	case NL80211_CMD_RADAR_DETECT: {
 		enum nl80211_radar_event event_type;
@@ -985,10 +1233,27 @@ static int print_event(struct nl_msg *msg, void *arg)
 	case NL80211_CMD_DEL_NAN_FUNCTION:
 		parse_nan_term(tb);
 		break;
-	case NL80211_CMD_NAN_MATCH: {
+	case NL80211_CMD_NAN_MATCH:
 		parse_nan_match(tb);
 		break;
-	}
+	case NL80211_CMD_NEW_PEER_CANDIDATE:
+		parse_new_peer_candidate(tb);
+		break;
+	case NL80211_CMD_NEW_INTERFACE:
+	case NL80211_CMD_SET_INTERFACE:
+	case NL80211_CMD_DEL_INTERFACE:
+		parse_recv_interface(tb, gnlh->cmd);
+		break;
+	case NL80211_CMD_STA_OPMODE_CHANGED:
+		parse_sta_opmode_changed(tb);
+		break;
+	case NL80211_CMD_STOP_AP:
+		printf("stop ap\n");
+		break;
+	case NL80211_CMD_CH_SWITCH_STARTED_NOTIFY:
+	case NL80211_CMD_CH_SWITCH_NOTIFY:
+		parse_ch_switch_notify(tb, gnlh->cmd);
+		break;
 	default:
 		printf("unknown event %d (%s)\n",
 		       gnlh->cmd, command_name(gnlh->cmd));
@@ -1172,5 +1437,5 @@ static int print_events(struct nl80211_state *state,
 TOPLEVEL(event, "[-t|-r] [-f]", 0, 0, CIB_NONE, print_events,
 	"Monitor events from the kernel.\n"
 	"-t - print timestamp\n"
-	"-r - print relative timstamp\n"
+	"-r - print relative timestamp\n"
 	"-f - print full frame for auth/assoc etc.");
